/************************************************************************/
/*									*/
/*	var2.c								*/
/*									*/
/*	Copyright (c) 1992, Research Systems Inc.  All rights reserved.	*/
/*									*/
/*	Test automatic type conversion, statically allocated variables	*/
/*									*/
/************************************************************************/


/*	All IDL server clients need to include 'idlrpc.h'		*/

#include        <stdio.h>
#include        <ctype.h>
#include        <rpc/rpc.h>
#include	"export.h"
#include	"rpc_idl.h"
#include	"string.h"

CLIENT		* client;	/*	opaque connection mechanism	*/

main(argc, argv)
     int	argc;
     char	*argv[];
{
    char	*hostname	= NULL;
    
    /*	Set up connection with IDL server. If there is a command	*/
    /*	line argument, assume that it is the hostname where the IDL	*/
    /*	server is running.						*/

    if(argc == 2)
	hostname	= argv[1];
    
    client	= register_idl_client( 0, hostname, NULL );
    
    /*	if client is NULL, then there is no IDL server running		*/

    if( client == NULL ) {
	printf("Can't find server.\n");
	exit(1);
    }

    /*	Perform testing with scalar variables.				*/

    test_conversions();
    test_statics();

    /*	Disconnect from server						*/

    unregister_idl_client( client );
}


static	void set_var(var)
    varinfo_t	* var;
{
    if( set_idl_variable( client, var ) != 1 ) {
	printf("set_idl_variable for %s failed\n", var->Name );
	exit(1);
    }
}

static	void get_var(name, var, type)
    char	* name;
    varinfo_t	* var;
    int		type;
{
    if( get_idl_variable( client, name, var, type ) != 1 ) {
	printf("get_idl_variable for %s failed\n", name );
	exit(1);
    }
}

static	void	send_cmd(cmd)
    char	*cmd;
{
    int		ret_code;
    if( (ret_code = send_idl_command( client, cmd )) != 1 ) {
	printf("Error %d encountered processing command \"%s\"\n",
	       ret_code, cmd );
    }
}

/************************************************************************/
/*									*/
/*	print_var							*/
/*									*/
/*	print contents of a variable.  Print only the first 10 elements	*/
/*	of array variables. This function only handles LONG,FLOAT,	*/
/*	DOUBLE and STRING arrays. It could easily be extented.		*/
/*									*/
/************************************************************************/

print_var( v )
varinfo_t	*v;
{
    VARIABLE	*var;
    UCHAR	*b;
    short	*ip;
    LONG	*l;
    float	*f;
    double	*d;
    STRING	*s;
    COMPLEX	*c;
    int		i;

    static	char *types[] = { "Undef", "Byte", "Int", "Long",
				      "Float", "Double", "Complex", "String",
				      "Struct???" };


    /*	Print the name and type of the variable				*/
    
    printf("Name: %s\n", v->Name );
    var	= v->Variable;
    printf("Type = %s\n", types[var->type ] );

    printf("Value = ");

    /*	If its an array, print the first 10 elements			*/
    /*	Note: This will do bad things if the array doesn't have at	*/
    /*	least 10 elements.						*/
    /*	This only prints a subset of the available types		*/
    
    if( var->flags & V_ARR ) {

	switch( var->type ) {
	  case TYP_BYTE:
	    for( b = (UCHAR*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %d", (int)b[i] );
	    break;
	  case TYP_INT:
	    for( ip = (short*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %ld", ip[i] );
	    break;
	  case TYP_LONG:
	    for( l = (LONG*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %ld", l[i] );
	    break;
	  case TYP_FLOAT:
	    for( f = (float*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %f", f[i] );
	    break;
	  case TYP_DOUBLE:
	    for( d = (double*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %lf", d[i] );
	    break;
	  case TYP_STRING:
	    for( s = (STRING*)var->value.arr->data, i = 0; i < 10; ++i ) {
		printf(" '%s'", STRING_STR( &s[i] ) );
	    }
	    break;
	  case TYP_COMPLEX:
	    for( c = (COMPLEX*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" (%f,%f)", c[i].r, c[i].i );
	    break;
	  default:
	    printf("print_var: can't print arrays of type %d",var->type);
	}
	printf("...\n" );
    }

    /*	Otherwise, print the value. UNDEF's have no value		*/
    else {
	switch( var->type ) {

	  case TYP_UNDEF:	printf("<Nil>\n");		break;
	  case TYP_BYTE:	printf("%d\n", var->value.c );	break;
	  case TYP_INT:		printf("%d\n", var->value.i );	break;
	  case TYP_LONG:	printf("%ld\n", var->value.l );	break;
	  case TYP_FLOAT:	printf("%f\n", var->value.f );	break;
	  case TYP_DOUBLE:	printf("%lf\n", var->value.d );	break;
	  case TYP_COMPLEX:	printf("(%f %f)\n",
				        var->value.cmp.r, var->value.cmp.i );
	    break;
	  case TYP_STRING:	printf("%s\n", STRING_STR(&var->value.str) );
	    break;
	  default:
	    printf("<unprintable>\n");	/* Struct???			*/
	    break;
	}
    }
}


/************************************************************************/
/*									*/
/*	test_statics							*/
/*									*/
/*	Build a few static variables and send them to and fro. Verify	*/
/*	that we: 1) Don't clobber memory we don't have. 2) don't free	*/
/*	static memory.							*/
/*									*/
/************************************************************************/
test_statics()
{
    varinfo_t	array;
    LONG	arr_dim[1];
    LONG	arr_buf[10000];
    LONG	i;

    /*	array = lonarr(10000)						*/
    bzero( &array, sizeof(array) );

    /*	Store array dimensions						*/
    arr_dim[0]	= 10000;

    /*	Fill varinfo_t structure. Send to server.			*/
    v_fill_array( &array, "ARRAY", TYP_LONG, 1, arr_dim,
		 (UCHAR*)arr_buf, sizeof(arr_buf) );

    /*	Build an array in the server and receive it 1000 times		*/
    send_cmd("FOO=LONG(DIST(100))");

    /*	End of current memory						*/

    printf("SBRK=%X\n ", sbrk(0));
    for(i = 0L; i < 1000L; ++i) {
	fprintf(stdout, "\b%c", "|/-\\"[i&3]);
	fflush(stdout);
	if(get_idl_variable( client, "FOO", &array, 0) != 1) exit(2);
    }

    printf("\n\nSBRK now = %X\n", sbrk(0));
    print_var( &array );

    if( (LONG)array.Variable->value.arr->data != (LONG)arr_buf ) {
	printf("Data reallocated!. %X vs %X\n", array.Variable->value.arr->data, arr_buf);
	exit(2);
    }
}


/************************************************************************/
/*									*/
/*	test_conversions						*/
/*									*/
/*									*/
/************************************************************************/
test_conversions()
{
    varinfo_t	array;
    varinfo_t	scalar;
    varinfo_t	result;
    LONG	arr_dim[1];
    float	arr_buf[10000];
    int		i;

    /*	array = findgen(10000)						*/
    bzero( &array, sizeof(array) );
    bzero( &scalar, sizeof(scalar) );    

    /*	Generate data							*/
    for(i=0; i<10000; ++i)
	arr_buf[i] = (float)i;

    /*	Store array dimensions						*/
    arr_dim[0]	= 10000;

    /*	Fill varinfo_t structure. Send to server.			*/
    v_fill_array(&array, "ARRAY", TYP_FLOAT, 1, arr_dim, (UCHAR*)arr_buf, 0);
    set_var( &array );

    /*	Build a scalar variable too.					*/
    v_make_int( &scalar, "SCALAR", 12*256+100 /* == 3172 (0xC64) */ );	
    set_var( &scalar );

    /*	Verify variables arrival in server.				*/
    send_cmd("help,array");
    send_cmd("help,scalar");

    bzero( &result, sizeof(result) );

    /*	Read and convert scalar to all types				*/

    if(get_idl_variable( client, "SCALAR", &result, 0) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_BYTE) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_INT) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_LONG) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_FLOAT) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_DOUBLE) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_COMPLEX) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "SCALAR", &result, TYP_STRING) != 1) exit(2);
    print_var( &result );


    /*	Read and convert array to all types				*/

    if(get_idl_variable( client, "ARRAY", &result, 0) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_BYTE) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_INT) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_LONG) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_FLOAT) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_DOUBLE) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_COMPLEX) != 1) exit(2);
    print_var( &result );

    if(get_idl_variable( client, "ARRAY", &result, TYP_STRING) != 1) exit(2);
    print_var( &result );

    free_idl_var( &result );
}

